@@ -21,8 +21,13 @@ from jupyter_dash import JupyterDash
 import dash_core_components as dcc
 import dash_html_components as html
 from dash.dependencies import Input, Output, State
+import dash_table
+import plotly.express as px
 import webbrowser
 
+from macrodemos.common_components import app_parameter, app_parameter_row, editable_cell_format
+
+
 import numpy as np
 from numpy.polynomial import Polynomial as P
 import pandas as pd
@@ -33,6 +38,9 @@ from statsmodels.tsa.arima_process import ArmaProcess # MODELO TEORICO
 from statsmodels.tsa.stattools import acf, pacf # autocorrelation and partial autocorrelation
 
 
+
+
+
 # Esta parte controla asuntos de estética de la página
 external_stylesheets = ['https://codepen.io/chriddyp/pen/bWLwgP.css']
 
@@ -138,6 +146,7 @@ class ARMA(ArmaProcess):
         self.show_estimates = False
         self.estimates = {'repr': ''}
         self.estimated = False
+        self.estimated_table = []
 
 
 
@@ -210,9 +219,23 @@ class ARMA(ArmaProcess):
             self.estimates['arroots'] = np.array([1/x for x in res.arroots])
             self.estimates['maroots'] = np.array([1 / x for x in res.maroots])
             self.estimated = True
+
+            res1 = res.summary().tables[1]
+            result = pd.DataFrame(res1.data[1:], columns=res1.data[0])
+            result = result.iloc[:, [0, 1, 4]]
+            result.rename(columns={'': 'param'}, inplace=True)
+
+
+            new_estimate = dash_table.DataTable(
+                data=result.to_dict('records'),
+                columns=[{'name': i, 'id': i} for i in result.columns])
+            self.estimated_table = new_estimate.data
+            print(self.estimated_table)
+
         except:
             self.estimated = False
             self.estimates['repr'] = 'The model could not be estimated'
+            self.estimated_table = []
 
     def print2moments(self):
         return f'$E(y_t) = {self.mean:g}, V(y_t) = {self.variance:g}$' if self.isstationary else 'This process is not stationay!'
@@ -255,53 +278,76 @@ class ARMA(ArmaProcess):
         return results
 
     def plot_correlogram(self, lags):
-        results = dict()
-        results['layout'] = {'title': 'Autocorrelations'}
-
-        if self.isstationary:
-            results['data'] = [{'y': self.acf(lags=lags), 'type': 'bar', 'name': 'actual'}]
-        else:
-            results['data'] = [{'y': np.zeros(lags), 'type': 'bar', 'name': 'not defined'}]
+        lags += 1
 
+        df = pd.DataFrame({'actual': self.acf(lags=lags)} if self.isstationary else {'not defined':np.zeros(lags)})
         if self.show_estimates:
-            self.estimates['acf'] = acf(self.simulated_data, unbiased=True, nlags=lags, fft=False)
-            results['data'].append({'y': self.estimates['acf'], 'type': 'bar', 'name': 'estimated'})
+            self.estimates['acf'] = acf(self.simulated_data, unbiased=True, nlags=lags-1, fft=False)
+            df['estimated'] = self.estimates['acf']
+
+        fig = px.bar(df,
+                     barmode='group',
+                     title='Autocorrelations',
+                     template='simple_white')
+
+        fig.update_layout(
+            legend_orientation='h',
+            xaxis_title='Lag',
+            yaxis_title=r'$\rho$'
+        )
+        return fig
 
-        return results
 
     def plot_partial_correlogram(self, lags):
-        results = dict()
-        results['layout'] = {'title': 'Partial Autocorrelations'}
-
-        if self.isstationary:
-            results['data'] = [{'y': self.pacf(lags=lags), 'type': 'bar', 'name': 'actual'}]
-        else:
-            results['data'] = [{'y': np.zeros(lags), 'type': 'bar', 'name': 'not defined'}]
+        lags += 1
 
+        df = pd.DataFrame({'actual': self.pacf(lags=lags)} if self.isstationary else {'not defined':np.zeros(lags)})
         if self.show_estimates:
-            self.estimates['pacf'] = pacf(self.simulated_data, nlags=lags)
-            results['data'].append({'y': self.estimates['pacf'], 'type': 'bar', 'name': 'estimated'})
+            self.estimates['acf'] = acf(self.simulated_data, unbiased=True, nlags=lags-1, fft=False)
+            df['estimated'] = self.estimates['acf']
+
+        fig = px.bar(df,
+                     barmode='group',
+                     title='Partial Autocorrelations',
+                     template='simple_white')
+
+        fig.update_layout(
+            legend_orientation='h',
+            xaxis_title='Lag',
+            yaxis_title=r'$\rho$'
+        )
+        return fig
+
 
-        return results
 
     def plot_process(self):
-        y = self.simulated_data.values
-        t = np.arange(y.size)
-        results = dict()
-        results['data'] = [{'x': t, 'y': y, 'type': 'line','name': 'actual'},
-                           {'x': [0, t[-1]], 'y': [self.mean, self.mean], 'type':'line','name':r'$\mu$'}]
-        results['layout'] = {'title': 'Simulated Data'}
-        if self.show_estimates and self.estimated:
-            results['data'].insert(1, {'x': t, 'y': self.estimates['fitted'], 'type': 'line','name': 'fitted'})
+        df = pd.DataFrame({'actual':self.simulated_data.values,
+                           r'$\mu$':self.mean})
 
-        """process = go.Figure()
-        process.add_trace(go.Scatter(x=t, y=y, name='actual'))
         if self.show_estimates and self.estimated:
-            process.add_trace(go.Scatter(x=t, y=self.estimates['fitted'], name='fitted'))
-        process.add_trace(go.Scatter(x=[0, t[-1]], y=[self.mean, self.mean], name=r'$\mu$'))
-        process.update_layout(title_text='Simulated Data')
-        """
-        return results #process
+            df['fitted'] = self.estimates['fitted']
+
+        fig = px.line(
+            data_frame=df,
+            title='Simulated Data',
+            template='simple_white'
+        )
+        fig.update_layout(
+            legend_orientation='h',
+            xaxis_title='Period',
+            yaxis_title=r'$y_t$',
+        )
+
+        fig.add_annotation(
+            x=0.2,
+            y=1.05,
+            xref="paper",
+            yref="paper",
+            text=str(self),
+            showarrow=False
+        )
+
+        return fig
 
     def plot_ar_roots(self):
         results = dict()
@@ -372,17 +418,26 @@ class ARMA(ArmaProcess):
         return results
 
     def plot_irf(self, horizon):
-        y = self.impulse_response(horizon)
-        x = np.arange(y.size)
-        result = dict()
-        result['data'] = [{'x': x, 'y': y, 'type': 'bar', 'name': 'actual'}]
-        result['layout'] = {'title': 'Impulse Response Function'}
+
+        df = pd.DataFrame({'actual':self.impulse_response(horizon)})
 
         if self.show_estimates and self.estimated:
-            temp = ARMA(self.estimates['c'], self.estimates['phi'], self.estimates['theta']).impulse_response(horizon)
-            x = np.arange(temp.size)
-            result['data'].append({'x': x, 'y': temp, 'type': 'bar', 'name': 'estimated'})
-        return result
+            df['estimated'] = ARMA(self.estimates['c'],
+                                   self.estimates['phi'],
+                                   self.estimates['theta']).impulse_response(horizon)
+
+
+        fig = px.bar(df,
+                     barmode='group',
+                     title='Impulse Response Function',
+                     template='simple_white')
+
+        fig.update_layout(
+            legend_orientation='h',
+            xaxis_title='Lag',
+            yaxis_title='IRF'
+        )
+        return fig
 
 
 
@@ -394,22 +449,13 @@ class ARMA(ArmaProcess):
 mathjax = ["https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.4/MathJax.js?config=TeX-MML-AM_CHTML"]  # to display math
 omega = np.linspace(0, np.pi, 121)  # frequencies for spectral plot
 
-def app_parameter(label, appid, inputtype, value, size='20'): # to make input parameter
-    return html.Th(label), html.Th(dcc.Input(id=appid, type=inputtype, value=value, size=size))
-
-def app_parameter_row(label, appid, inputtype, value, size='20'): # to make input parameters
-    return html.Tr([*app_parameter(label, appid, inputtype, value, size)])
-
-
 
 app = JupyterDash(__name__,external_stylesheets=external_stylesheets, external_scripts=mathjax)
 
 #======DESIGN THE APP===============
 
 app.layout = html.Div(children=[
-    html.H2(id='title',
-            style={'textAlign': 'center', 'color': colors['text']}),
-    html.P(id='estimated',
+    html.H2("The ARMA model",
             style={'textAlign': 'center', 'color': colors['text']}),
     html.Div(children=[html.H4("ARMA Parameters"),
                        html.Table(children=[
@@ -431,8 +477,16 @@ app.layout = html.Div(children=[
                                                          value='No'))]),
                        ],
                        ),
-                       html.Div(id='p,q', children = [html.Tr([*app_parameter('p', 'p', 'text', '0', '5'),
-                                                               *app_parameter('q', 'q', 'text', '0', '5')])]),
+                       html.Div(id='p,q', children=[html.Tr([*app_parameter('p', 'p', 'text', '0', '5'),
+                                                               *app_parameter('q', 'q', 'text', '0', '5')],
+                                                              style={'border':'3px dashed SteelBlue'}),
+                                                    dash_table.DataTable(
+                                                        id='estimate_table',
+                                                        columns=[{'name': '  ', 'id': 'param'},
+                                                                 {'name': 'coef', 'id': 'coef'},
+                                                                 {'name': 'p-value', 'id': 'P>|z|'}],
+                                                        data=[])
+                                                    ]),
                        html.Hr(),
                        html.Button('PLOT', id='execute',style={'textAlign': 'center', 'backgroundColor': colors['buttons']}),
                        html.P('ARMA PROCESS DEMO', style={'textAlign': 'center', 'color': colors['text'],'marginTop':500}),
@@ -470,17 +524,17 @@ style={'backgroundColor': colors['background']})
 
 
 @app.callback(
-    [Output('title', 'children'),
-     Output('plot-process', 'figure'),
-     Output('plot-correlogram', 'figure'),
-     Output('plot-partial-correlogram', 'figure'),
-     Output('plot-irf', 'figure'),
-     Output('plot-spectrum', 'figure'),
-     Output('plot-ar-roots', 'figure'),
-     Output('plot-ma-roots', 'figure'),
-     Output('moments', 'children'),
-     Output('moments', 'style'),
-     Output('estimated', 'children'),
+    [
+        Output('plot-process', 'figure'),
+        Output('plot-correlogram', 'figure'),
+        Output('plot-partial-correlogram', 'figure'),
+        Output('plot-irf', 'figure'),
+        Output('plot-spectrum', 'figure'),
+        Output('plot-ar-roots', 'figure'),
+        Output('plot-ma-roots', 'figure'),
+        Output('moments', 'children'),
+        Output('moments', 'style'),
+        Output('estimate_table','data')
      ],
     [Input('execute', 'n_clicks')],
     [State('c', 'value'),
@@ -505,7 +559,6 @@ def update_ARMA_parameters(n_clicks, c, phi, theta, sigma2, periods, lags, horiz
         Y.estimate(int(p), int(q))
 
     return [
-        str(Y),
         Y.plot_process(),
         Y.plot_correlogram(lags=int(lags)),
         Y.plot_partial_correlogram(lags=int(lags)), #FIXME
@@ -515,7 +568,7 @@ def update_ARMA_parameters(n_clicks, c, phi, theta, sigma2, periods, lags, horiz
         Y.plot_ma_roots(),
         Y.print2moments(),
         style,
-        Y.estimates['repr']
+        Y.estimated_table
     ]
 
 
@@ -531,15 +584,25 @@ def show_pq_options(showestimates, phi, theta):
     theta = validate_lists_of_values(theta)
     Y = ARMA(0, phi, theta, 1.0)
     p, q = Y.p, Y.q
-    children = html.Tr([*app_parameter('p', 'p', 'text', p, '4'),
-                        *app_parameter('q', 'q', 'text', q, '4')])
-    style = {'display': 'none'} if (showestimates == 'No') else {}
+    children = [html.Tr([*app_parameter('p', 'p', 'text', p, '4'),
+                        *app_parameter('q', 'q', 'text', q, '4')],
+                       style={'border':'3px dashed SteelBlue'}),
+                dash_table.DataTable(
+                    id='estimate_table',
+                    columns=[{'name': '  ', 'id': 'param'},
+                             {'name': 'coef', 'id': 'coef'},
+                             {'name': 'p-value', 'id': 'P>|z|'}],
+                    data=[],
+                    style_cell=editable_cell_format
+                )
+                ]
+    style = {'display': 'none'} if (showestimates == 'No') else {'border': '3px dashed SteelBlue'}
     return children, style
 
 
 
 
-def ARMA_demo(colab=True):
+def ARMA_demo(colab=False):
     if colab:
         app.run_server(mode='external')
     else:
