@@ -41,6 +41,7 @@ from scipy.linalg import toeplitz
 
 import requests
 import bs4 as bs
+import os
 
 pd.core.common.is_list_like = pd.api.types.is_list_like
 
@@ -121,14 +122,8 @@ def read_IFS(indicator, ref_area, frequency='M', startPeriod=None, endPeriod=Non
 """==========================================================================================================
 DATAFILE contains the raw data to be filtered
 """
-DATAFILE = r"data\IFS_GDP.xlsx"
-
-try:
-    AVAILABLE_COUNTRIES = pd.read_excel(DATAFILE, 'Countries')
-except:
-    DATAFILE = 'macrodemos\\' + DATAFILE
-    AVAILABLE_COUNTRIES = pd.read_excel(DATAFILE, 'Countries')
-
+DATAFILE = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data', 'IFS_GDP.xlsx')
+AVAILABLE_COUNTRIES = pd.read_excel(DATAFILE, 'Countries')
 
 AVAILABLE_COUNTRIES.set_index(['ISO', 'NAME'], inplace=True)
 AVAILABLE_COUNTRIES = AVAILABLE_COUNTRIES[AVAILABLE_COUNTRIES.sum(axis=1)>0]
@@ -534,7 +529,7 @@ def update_plots(n_clicks, indicator, whichFilters, lambda_, K, pL, pH, cfpL, cf
 This block executes the app
 """
 
-def filters_demo(colab=True):
+def filters_demo(colab=False):
     if colab:
         app.run_server(mode='external')
     else:
